<?php

App::uses('EvTemplatesAppController', 'EvTemplates.Controller');
App::uses('Folder', 'Utility');
App::uses('TemplateUtil', 'Lib');

class TemplatesController extends EvTemplatesAppController
{
	/**
	 * Prepare data for populating admin dropdowns.
	 */
	protected function _adminPopulateLookups()
	{
		parent::_adminPopulateLookups();

		$templates = $this->getSelectableTemplates();
		$attachmentBlockFields = $this->getBlockFields();
		$helperFunctions = $this->getHelperFunctions();

		$this->set(compact('templates', 'attachmentBlockFields', 'helperFunctions'));
	}


	/**
	 * get the list of available methods on the Templates Utility
	 *
	 * @return array 	array of available methods
	 */
	protected function getHelperFunctions()
	{
		$utilReflection = new \ReflectionClass('TemplateUtil');
		$utilMethods = $utilReflection->getMethods(ReflectionMethod::IS_PUBLIC);

		$methods = array('N/A' => 'N/A');
		foreach ($utilMethods as $item) {

			$methods[$item->name] = $item->name;
		}

		return $methods;
	}


	/**
	 * return array of block fields available (alt, caption etc..)
	 *
	 * @return array 	array of available fields
	 */
	protected function getBlockFields()
	{
		return array(
			'ImageBlock' => array(
				array(
					'value' => 'alt',
					'name' => 'Alt',
					'class' => 'protect-value' // sets class on the input checkbox to prevent js cloneable from removing value
				),
				array(
					'value' => 'caption',
					'name' => 'Caption',
					'class' => 'protect-value' // sets class on the input checkbox to prevent js cloneable from removing value
				),
				array(
					'value' => 'link_text',
					'name' => 'Link Text',
					'class' => 'protect-value' // sets class on the input checkbox to prevent js cloneable from removing value
				),
				array(
					'value' => 'link_url',
					'name' => 'Link Url',
					'class' => 'protect-value' // sets class on the input checkbox to prevent js cloneable from removing value
				)
			),
			'DocumentBlock' => array(
				array(
					'value' => 'alt',
					'name' => 'Alt',
					'class' => 'protect-value' // sets class on the input checkbox to prevent js cloneable from removing value
				),
				array(
					'value' => 'caption',
					'name' => 'Caption',
					'class' => 'protect-value' // sets class on the input checkbox to prevent js cloneable from removing value
				)
			)
		);
	}


	/**
	 * find all the templates within the view folder
	 *
	 * @return array 	array of templates to populate select
	 */
	protected function getSelectableTemplates()
	{
		// search for templates
		$templates = array();

		$templateDir = 'Templates';
		$viewDir = APP . 'View' . DS;
		$dir = new Folder($viewDir . $templateDir);
		$files = $dir->findRecursive('.*\.ctp');

		foreach ($files as $file) {

			$filePath = str_replace($viewDir . $templateDir . DS, '', $file);
			$filename = substr($filePath, 0, -4);

			$templates[$templateDir . DS . $filename] = $filePath;
		}

		return $templates;
	}


	/**
	 * Prepare the admin form fields for the menu item form
	 * @return array
	 */
	protected function _adminFormFields() {

		$Model = $this->{$this->modelClass};

		$fields = parent::_adminFormFields();

		$fields[$Model->alias . '.template']['type'] = 'select';
		$fields[$Model->alias . '.helperFunction']['type'] = 'select';

		return $fields;
	}


	/**
	 * redefine admin_edit to set our own template
	 *
	 */
	public function admin_edit($id = null)
	{
        if (
            ! empty($this->request->data) &&
            (
                $this->request->is('post') ||
                $this->request->is('put')
            )
        ) {
        	// check for image / document blocks to delete
        	if (! empty($this->request->data['ImageBlock']['toDelete'])) {
        		$this->deleteBlocks('Image');
        	}

        	if (! empty($this->request->data['DocumentBlock']['toDelete'])) {
        		$this->deleteBlocks('Document');
        	}

			unset(
				$this->request->data['ImageBlock']['toDelete'],
				$this->request->data['DocumentBlock']['toDelete']
			);
	    }

		parent::admin_edit($id);

		$this->view = 'EvTemplates.Templates/admin_form';
	}


	/**
	 * delete any images / document blocks that have been marked for delete
	 *
	 * @param 	string 		Type of block to delete
	 */
	protected function deleteBlocks($type)
	{
		$ids = explode(',', $this->request->data[$type . 'Block']['toDelete']);
		$ids = ArrayUtil::clearArray($ids);

		$Model = EvClassRegistry::init('EvTemplates.' . $type . 'Block');
		$Model->deleteAll(
			array(
				$type . 'Block.id' => $ids
			),
			true,
			true
		);
	}


	/**
	 * called via ajax
	 * given the template ID get all the fields and process in template
	 * so we can fill out the data
	 *
	 * @param 	int 	Template id
	 * @return 	string 	processed html containing the fields
	 */
	public function ajax_getFieldsForTemplate($templateId = null)
	{
		$this->view = 'EvTemplates.Templates/fields';
		$this->layout = 'ajax';

		if (empty($templateId)) {
			return;
		}

		$template = $this->Template->readForView($templateId);

		$this->set('template', $template);
	}
}
